# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2015 Free Software's Seed
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++


class TipoContratoController < ApplicationController

  before_filter :elementos_filtrado, :only => [ :listado ]

  def index
    redirect_to :action => :listado 
  end

  # en agentes: establece los parametros de ordenación
  def ordenado
    session[:tipo_contrato_asc_desc] = params[:asc_desc] ? params[:asc_desc] : "ASC"
    session[:tipo_contrato_orden] = params[:orden] ? params[:orden] : "nombre"
    redirect_to :action => :listado
  end

  # en agentes: establece los parametros de filtro
  def filtrado
    session[:tipo_contrato_filtro_creador] = params[:filtro][:creador] if params[:filtro][:creador]
    redirect_to :action => :listado
  end

  def elementos_filtrado
    session[:tipo_contrato_asc_desc] ||= "ASC"
    session[:tipo_contrato_orden] ||= "nombre"
    session[:tipo_contrato_filtro_creador] ||= "todos"

    @creador = Agente.find_by_id( session[:tipo_contrato_filtro_creador] ) if session[:tipo_contrato_filtro_creador].to_i > 0

    datos_formulario
    opciones_generales = { "todos" => _("Todos"), "general" => _("Administración General") }
    filtro_creador = opciones_generales.collect{|k,v| [v,k]} + @agentes
    
    @opciones_filtrado = [ {:rotulo => _("Seleccione creador"), :nombre => "creador", :opciones => filtro_creador} ]
    @accion_filtrado = {:action => :filtrado, :listado => :listado}
    @estado_filtrado = [ (@creador ? @creador.nombre : (opciones_generales[session[:tipo_contrato_filtro_creador]] || _("Todos")) ) ]
  end


	# en agentes y administracion: lista los tipos de contrato
  def listado
    elementos_filtrado 
    condiciones = {}
    condiciones[:agente_id] = @creador.id if @creador
    condiciones[:agente_id] = nil if session[:tipo_contrato_filtro_creador] == "general"
    condiciones[:agente_id] = [@agente.id, nil] if @agente && session[:tipo_contrato_filtro_creador] == "todos"

    @tipos_contrato = @paginado = TipoContrato.paginate(:conditions => condiciones,
		:order => (session[:tipo_contrato_orden] + " " + session[:tipo_contrato_asc_desc] ),
		:page => (params[:format]=='xls' ? nil : params[:page]), :per_page => (params[:format_xls_count] || session[:por_pagina]) )

    @formato_xls = @tipos_contrato.total_entries
    respond_to do |format|
      format.html
      format.xls do
        @tipo = "tipo_contrato"
        @objetos = @tipos_contrato
        nom_fich = "tipos_contrato_" + Time.now.strftime("%Y-%m-%d")
        render 'comunes_xls/listado', :xls => nom_fich, :layout => false
      end
    end
  end

	# en agentes y administracion: prepara el formulario de crear o editar
  def editar_nuevo
    @tipo_contrato = ( TipoContrato.where(agente_id: @agente.id).find_by_id(params[:id]) || TipoContrato.new(agente_id: @agente.id) ) if @agente
    @tipo_contrato = ( TipoContrato.find_by_id(params[:id]) || TipoContrato.new ) if administracion? 
    datos_formulario
    render (:update) { |page| page.formulario(:partial => "formulario", :update => params[:update]) }
  end

	# en agentes y administracion: modifica o crea un tipo de contrato 
  def modificar_crear
    @tipo_contrato = ( TipoContrato.where(agente_id: @agente.id).find_by_id(params[:id]) || TipoContrato.new(agente_id: @agente.id) ) if @agente
    @tipo_contrato = ( TipoContrato.find_by_id(params[:id]) || TipoContrato.new ) if administracion? 
    @tipo_contrato.update_attributes params[:tipo_contrato]
    if @tipo_contrato.errors.empty?
      # Si es uno ya existente, modifica la linea
      render(:update) { |page|   page.modificar :update => params[:update], :partial => "tipo_contrato" , :mensaje => { :errors => @tipo_contrato.errors } } if params[:id]
      # Si es uno nuevo lo incluye en la parte superior del listado
      render :update do |page|
        page.show "nuevos_tipos_contrato"
        page.modificar :update => "tipo_contrato_nuevo_" + params[:i], :partial => "nuevo_tipo_contrato", :mensaje => { :errors => @tipo_contrato.errors }
        page.replace "formulario_anadir_anadir", :inline => "<%= anadir(:url => {:action => 'editar_nuevo', :i => (params[:i].to_i + 1).to_s, :update => 'formulario_anadir'}) %>"
      end unless params[:id] 
    else
      datos_formulario
      render(:update) { |page| page.recargar_formulario :partial => "formulario", :mensaje => {:errors => @tipo_contrato.errors} }
    end 
  end

	# en agentes y administracion: elimina un tipo de contrato 
  def eliminar
    @tipo_contrato = TipoContrato.where(agente_id: @agente.id).find_by_id(params[:id]) if @agente
    @tipo_contrato = TipoContrato.find_by_id(params[:id]) if administracion? 
    @tipo_contrato.destroy if @tipo_contrato
    render (:update) {|page| page.eliminar :update => params[:update], :mensaje => {:errors => @tipo_contrato.errors, :eliminar => true}}
  end

 private
  def datos_formulario
    @agentes = Agente.where(implementador: true, socia_local: false).order(:nombre).collect{|p| [p.nombre, p.id]} if administracion? 
    @agentes = [[@agente.nombre, @agente.id]] if @agente
  end

  def administracion? 
    params[:seccion] == "administracion"
  end
end

